SUBROUTINE fourierkraft(FORCES,NATOM,NTYPEATOM,NSUB,NATOMTOT,QPCOM,antalmod,MASS,SSZ,FFORCE)
      IMPLICIT NONE
      EXTERNAL :: invert
      DOUBLE PRECISION, INTENT(IN) ::FORCES(NATOMTOT,3),MASS(10),QPCOM(NSUB,3)
      INTEGER, INTENT(IN) :: NATOM,NTYPEATOM,NSUB,NATOMTOT,SSZ(3),antalmod
      COMPLEX*16,INTENT(OUT) :: FFORCE(NSUB,antalmod)
      
      ! NATOM = number of atoms in non-supercell
      ! NTYPEATOM = number of different ions
      ! NSUB = SSZ(1)*SSZ(2)*SSZ(3)= number of atoms in each sub-lattice
      ! NATOMTOT = SSZ(1)*SSZ(2)*SSZ(3)*NATOM = total number of atoms in supercell
      ! MASSUB(i) = mass of ions belonging to sublattice i.
      
      INTEGER :: i,j,l,m,NMASS(NTYPEATOM),n,jm,n1,n2,n3,n4,n5,n6
      DOUBLE PRECISION ::R(3),RR(NATOMTOT,3),K(3),MATR(3,3),MASSUB(NATOM),MATR2(3,3),RECGITTER(3,3),LATT
      COMPLEX*16 :: IM,FSUB(NATOM,NSUB,3)
      CHARACTER(LEN=3) :: DUMMY
      DOUBLE PRECISION, PARAMETER :: pi =3.1415926535897932384626433832795028841970d0
      IM = CMPLX(0.0d0,1.0d0)
      
      !Here the bravais lattice vectors and the atom positions
      !are read into the routine

      OPEN(10,FILE='POSCAR',STATUS='OLD',ACTION='READ')
      READ(10,*) DUMMY
      READ(10,*) LATT
      READ(10,*) MATR(1,1),MATR(1,2),MATR(1,3)
      READ(10,*) MATR(2,1),MATR(2,2),MATR(2,3)
      READ(10,*) MATR(3,1),MATR(3,2),MATR(3,3)
      READ(10,*) NMASS
      READ(10,*) DUMMY
      DO j=1,NATOMTOT
        READ(10,*) RR(j,1),RR(j,2),RR(j,3)
      ENDDO
      close(10)

      ! Here the masses of the different atom types are put into
      ! the array MASSUB
      jm = 0
      DO i=1,NTYPEATOM
        j= NMASS(i)/NSUB
        DO l=1,j
                MASSUB(l+jm) = MASS(i)
        ENDDO
        jm = j+jm
      ENDDO
      
      MATR = TRANSPOSE(MATR)

      ! Here the reciprocal lattice vectors are calculated:

      MATR2(:,1) = MATR(:,1)/(1.0d0*SSZ(1))
      MATR2(:,2) = MATR(:,2)/(1.0d0*SSZ(2))
      MATR2(:,3) = MATR(:,3)/(1.0d0*SSZ(3))

      call invert(MATR2,RECGITTER)

      RECGITTER = TRANSPOSE(RECGITTER)

      ! Here the fourier transform of the force is calculated for
      ! the different sub-lattices
      
      DO i = 1,NATOM
         DO n = 1,NSUB
                FSUB(i,n,1) = CMPLX(0.0d0, 0.0d0)
                FSUB(i,n,2) = CMPLX(0.0d0, 0.0d0)
                FSUB(i,n,3) = CMPLX(0.0d0, 0.0d0)
                K = QPCOM(n,:)
                K = MATMUL(RECGITTER,K)
                DO j=1,NSUB
                        !R = RR(j+(i-1)*NSUB,:)
                        R = RR(j,:) ! R runs only over 1:st sublattice basisvect = (0,0,0)
                                    ! see page 4 in program notes
                        R = MATMUL(MATR,R)
                        n1 = j+(i-1)*NSUB
                        FSUB(i,n,:)=FSUB(i,n,:)+(1.0d0/sqrt(MASSUB(i)))*FORCES(n1,:)* &
                        &exp(2.0d0*pi*im*DOT_PRODUCT(R,K))
                ENDDO
         ENDDO
      ENDDO

      ! Here the total fourier transform is calculated

      DO i = 1,NATOM
        DO n = 1,NSUB
               n1 = 1+(i-1)*3
               n2 = 2+(i-1)*3
               n3 = 3+(i-1)*3
               FFORCE(n,n1)=FSUB(i,n,1)
               FFORCE(n,n2)=FSUB(i,n,2)
               FFORCE(n,n3)=FSUB(i,n,3)
        ENDDO
      ENDDO
END SUBROUTINE fourierkraft
