SUBROUTINE getsymblock(NUMBATOM,NTYPEATOM,SSZ,NSYMREL,SYMBLOCK,SYMOPF)
! subroutine for extracting the symmetry relations 
! for the different 3x3 sub-matrices V(R+b_i-b_j) of
! the total 3NUMBATOM X 3NUMBATOM matrix.
! the out-put is the tensors SYMBLOCK and SYMOPF.
! SYMBLOCK(i,j,n,1) = k, index of supercell atomic position R(k)
! SYMBLOCK(i,j,n,2) = l, index of supercell atomic position R(l)
! SYMBLOCK(i,j,n,3) = m, index of the symmetry operation which is
! such that R(l) = S(m)R(k).
! SYMBLOCK(i,j,n,4) > 0 there is information stored, else no info.
! i = row index of subblock, j = column index of subblock.
IMPLICIT NONE
EXTERNAL :: getlatt
INTEGER, INTENT(IN) :: NUMBATOM,SSZ(3),NSYMREL,NTYPEATOM
INTEGER, INTENT(OUT) :: SYMBLOCK(NUMBATOM,NUMBATOM,NSYMREL,4)
DOUBLE PRECISION, INTENT(OUT) :: SYMOPF(NUMBATOM,48,3,3)
CHARACTER(LEN=3) :: DUMMY
DOUBLE PRECISION, ALLOCATABLE :: RO(:,:),R(:,:)
DOUBLE PRECISION :: base(NUMBATOM,3),A(3,3),LATTCONST
DOUBLE PRECISION ::PRIMLATT(3,3),RECLATT(3,3),MASS(10)
INTEGER :: SYMBL(NUMBATOM,NSYMREL,4)
DOUBLE PRECISION :: AA,TEMP,MATR(3,3)
INTEGER :: n,i,j,k,D1,D2,D3(3),read_stat,m,n1,n2,n3
INTEGER, ALLOCATABLE :: NA(:)
ALLOCATE(NA(NTYPEATOM))
DO i = 1,NUMBATOM
        DO j = 1,NSYMREL
                SYMBL(i,j,4) = -1
        ENDDO
ENDDO
DO i = 1,NUMBATOM
DO j = 1,NUMBATOM
DO k = 1,NSYMREL
SYMBLOCK(i,j,k,4) = -1
ENDDO
ENDDO
ENDDO
DO j=1,NUMBATOM
DO i = 1,48
        SYMOPF(j,i,:,:) = 0.0d0
ENDDO
ENDDO

! Here the atomic original atomic positions of the supercell file
! POSCAR are stored
n = SSZ(1)*SSZ(2)*SSZ(3)*NUMBATOM

ALLOCATE(RO(n,3))
ALLOCATE(R(n,3))
OPEN(10,FILE='POSCAR',STATUS='OLD',ACTION='READ')
READ(10,*) DUMMY
READ(10,*) LATTCONST
READ(10,*) A(1,1),A(1,2),A(1,3)
READ(10,*) A(2,1),A(2,2),A(2,3)
READ(10,*) A(3,1),A(3,2),A(3,3)
READ(10,*) NA
READ(10,*) DUMMY

DO i=1,n
        READ(10,*)RO(i,1),RO(i,2),RO(i,3)
ENDDO
close(10)

! Here the basis-vectors are extracted
DO i = 1,NUMBATOM
        n1 = (i-1)*SSZ(1)*SSZ(2)*SSZ(3)+1
        base(i,:) = RO(n1,:)
        !print*,'base(',i,')',base(i,1),base(i,2),base(i,3) 
ENDDO

! Here the tensor SYMBL is "filled"
DO j=1,NUMBATOM
        DO i=1,n
                R(i,:) = RO(i,:) - base(j,:)
        ENDDO
        OPEN(20,FILE='POSCAR',ACTION='WRITE')
        1 FORMAT(3(F20.15))
        WRITE(20,*)'Supercell file'
        WRITE(20,*)LATTCONST
        WRITE(20,FMT=1)A(1,1),A(1,2),A(1,3)
        WRITE(20,FMT=1)A(2,1),A(2,2),A(2,3)
        WRITE(20,FMT=1)A(3,1),A(3,2),A(3,3)
        WRITE(20,*)NA
        WRITE(20,*)'Direct'
        DO i=1,n
                WRITE(20,FMT=1)R(i,1),R(i,2),R(i,3)
        ENDDO
        close(20)

        ! Here the TRANSROT containing the symmetry iformation of column block j
        ! is created.
        ! The variables PRIMLATT,RECLATT,AA,D1,D2,TEMP,D3,MASS are all dummy,
        ! i.e not used by the subroutine getsymblock
        
        call getlatt(PRIMLATT,RECLATT,AA,D1,D2,TEMP,D3,MASS)

        OPEN(30,FILE='TRANSROT',STATUS='OLD',ACTION='READ')
        m = 0
        doread: DO i=1,NSYMREL
                m = m+1
                READ(30,*,IOSTAT = read_stat)SYMBL(j,m,1),SYMBL(j,m,2),SYMBL(j,m,3)
                IF ( read_stat .NE. 0 ) EXIT doread
                SYMBL(j,m,4) = 1
                READ(30,*)SYMOPF(j,SYMBL(j,m,3),1,1),SYMOPF(j,SYMBL(j,m,3),1,2),SYMOPF(j,SYMBL(j,m,3),1,3)
                READ(30,*)SYMOPF(j,SYMBL(j,m,3),2,1),SYMOPF(j,SYMBL(j,m,3),2,2),SYMOPF(j,SYMBL(j,m,3),2,3)
                READ(30,*)SYMOPF(j,SYMBL(j,m,3),3,1),SYMOPF(j,SYMBL(j,m,3),3,2),SYMOPF(j,SYMBL(j,m,3),3,3)
                ! put symmetry matrix in cartesian coord
                MATR = SYMOPF(j,SYMBL(j,m,3),:,:) 
                MATR = MATMUL(PRIMLATT,MATMUL(MATR,TRANSPOSE(RECLATT)))
                SYMOPF(j,SYMBL(j,m,3),:,:) = MATR
        ENDDO doread
        !print*,m
        close(30) 

        ! Here the SYMBL array is stored in the out array:
        ! SYMBLOCK(NUMBATOM,NUMBATOM,NSYMREL,4)
        
        DO i=1,NUMBATOM
                m = 0
                D1 = (i-1)*SSZ(1)*SSZ(2)*SSZ(3)+1
                D2 = i*SSZ(1)*SSZ(2)*SSZ(3)
                DO k = 1,NSYMREL
                        IF ( SYMBL(j,k,1) .GE. D1 .AND. SYMBL(j,k,1) .LE. D2 .AND. SYMBL(j,k,4) > 0) THEN
                                m= m+1        
                                SYMBLOCK(i,j,m,4) = 1
                                !Here we also rewrite the lattice inexes
                                !SYMBLOCK(i,j,m,1) and SYMBLOCK(i,j,m,2) so that 
                                !the "point" at the vectors R in the arguments: 
                                !R+base(i)-base(j)

                                SYMBLOCK(i,j,m,1) = SYMBL(j,k,1)-(i-1)*SSZ(1)*SSZ(2)*SSZ(3) 
                                SYMBLOCK(i,j,m,2) = SYMBL(j,k,2)-(i-1)*SSZ(1)*SSZ(2)*SSZ(3)
                                SYMBLOCK(i,j,m,3) = SYMBL(j,k,3)
                         ENDIF
                ENDDO
         ENDDO
ENDDO

! HERE THE ORIGINAL POSCAR FILE is restored:
OPEN(20,FILE='POSCAR',ACTION='WRITE')
WRITE(20,*)'Supercell file'
WRITE(20,*)LATTCONST
WRITE(20,FMT=1)A(1,1),A(1,2),A(1,3)
WRITE(20,FMT=1)A(2,1),A(2,2),A(2,3)
WRITE(20,FMT=1)A(3,1),A(3,2),A(3,3)
WRITE(20,*)NA
WRITE(20,*)'Direct'
DO i=1,n
   WRITE(20,FMT=1)RO(i,1),RO(i,2),RO(i,3)
ENDDO
close(20)

END SUBROUTINE getsymblock
